import pytest
import logging
from selenium.common import StaleElementReferenceException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import csv
from 预定系统.base.Broswer_driver import Browser_driver

def read_csv_data():
    """
    读取 CSV 文件数据
    """
    data = []
    try:
        with open('../测试数据/Login_Test.csv', mode='r', encoding='utf-8') as csvfile:
            csv_reader = csv.DictReader(csvfile)
            for row in csv_reader:
                data.append((row['username'], row['password'], row['verification_code']))
    except FileNotFoundError:
        logger.error("CSV 文件未找到，请检查文件路径")
        pytest.exit("CSV 文件未找到，请检查文件路径")
    except Exception as e:
        logger.error(f"出现错误: {e}")
        pytest.exit(f"出现错误: {e}")
    return data

# 设置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

@pytest.fixture(scope='session')
def driver():
    """
    初始化浏览器并打开指定URL
    """
    url = 'https://192.168.5.218'
    logging.info("'----------' 正在初始化浏览器 '----------'")
    try:
        driver = Browser_driver().browser_init()  # 初始化浏览器驱动
        logging.info("'----------' 浏览器初始化成功 '----------'")
    except Exception as e:
        logging.error(f"浏览器初始化失败: {e}")
        raise

    driver.implicitly_wait(10)  # 设置隐式等待时间为10秒
    driver.maximize_window()  # 最大化浏览器窗口
    logging.info("'----------' 正在打开URL '----------'")

    try:
        driver.get(url)
        logging.info(f"成功打开URL: {url}")
    except Exception as e:
        logging.error(f"打开URL失败: {e}")
        raise

    handle_ssl_warning(driver)  # 处理可能存在的SSL证书警告

    yield driver
    driver.quit()  # 在测试结束后关闭浏览器

def handle_ssl_warning(driver):
    """处理 SSL 证书警告"""
    try:
        logging.info("'----------' 正在处理SSL证书警告 '----------'")

        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="details-button"]'))
        ).click()

        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="proceed-link"]'))
        ).click()

        # 等待页面加载完成
        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.TAG_NAME, 'body'))
        )

        logging.info("'----------' SSL证书警告处理完成 '----------'")
    except Exception as e:
        logging.error(f"处理 SSL 证书警告遇到错误: {e}")
        raise

def _find_element(driver, by, value):
    """
    查找元素，并处理可能的异常
    """
    try:
        return WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((by, value))
        )
    except StaleElementReferenceException:
        return WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((by, value))
        )

def perform_login(driver, username, password, verification_code):
    """
    执行登录操作
    """
    try:
        # 清除并输入用户名
        username_input = _find_element(driver, By.XPATH,
                                       '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[1]/div/input')
        username_input.clear()
        username_input.send_keys(username)

        # 清除并输入密码
        password_input = _find_element(driver, By.XPATH,
                                       '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[2]/div/input')
        password_input.clear()
        password_input.send_keys(password)

        # 清除并输入验证码
        verification_input = _find_element(driver, By.XPATH,
                                           '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[3]/div[1]/div/input')
        verification_input.clear()
        verification_input.send_keys(verification_code)

        # 点击登录按钮
        login_button = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[4]/input')
        login_button.click()

        if username | password | verification_code == "":
            check_login_null(driver)
            logger.info("登录失败，必填项不能为空！断言成功！\n")

        elif check_login_failure(driver):
            logger.info("登录失败, 必填项错误！断言成功！\n")

        elif check_login_success(driver):
            logger.info("登录成功，断言成功！\n")
            return_to_login_page(driver)  # 返回登录页面
        # # 检查登录是否成功
        # if check_login_null(driver):
        #     logger.info("登录失败，必填项不能为空！断言成功！\n")
        #
        # elif check_login_success(driver):
        #     logger.info("登录成功，断言成功！\n")
        #     return_to_login_page(driver)  # 返回登录页面
        # elif check_login_failure(driver):
        #     logger.info("登录失败, 必填项错误！断言成功！\n")
        else:
            logger.error("登录状态未知，断言失败！\n")
            pytest.fail("登录状态未知，断言失败！")
    except StaleElementReferenceException as e:
        logger.error(f"元素已失效，重新查找... {e}\n")
        pytest.fail(f"元素已失效，重新查找... {e}")
    except Exception as e:
        logger.error(f"登录操作失败: {e}\n")
        pytest.fail(f"登录操作失败: {e}")

def return_to_login_page(driver):
    """
    返回登录页面
    """
    try:
        logout_button = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[1]/div/div[1]/div[2]/div[2]')
        logout_button.click()
        logging.info("已成功返回登录页面\n")
    except Exception as e:
        logger.error(f"返回登录页面失败: {e}\n")
        pytest.fail(f"返回登录页面失败: {e}")

def check_login_null(driver):
    """
    检查登录失败，必填项为空的情况
    """
    try:
        login_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, '/html/body/div[2]/p'))
        ).text

        assert "请输入" in login_element_text, "登录失败后未找到'请输入'提示语"
        return True
    except Exception as e:
        logger.error(f"登录失败检查失败: {e}")
        return False

def check_login_success(driver):
    """
    检查登录成功的情况
    """
    try:
        message_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, "//*[contains(text(), '欢迎')]"))
        ).text

        assert "欢迎 预定标准版测试" == message_element_text, "登录成功后找到'欢迎 预定标准版测试'提示语"

        driver.find_element(By.XPATH, '//*[@id="app"]/div/div[1]/div/img[3]').click()
        driver.refresh()
        driver.implicitly_wait(5)
        return True
    except Exception as e:
        logger.error(f"登录成功检查失败: {e}")
        return False

def check_login_failure(driver):
    """
    检查登录失败的情况
    """
    try:
        login_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[1]/div[1]'))
        ).text

        assert "账号登录" == login_element_text, "登录失败后未找到'账号登录'提示语"
        return True
    except Exception as e:
        logger.error(f"登录失败检查失败: {e}")
        return False

@pytest.mark.parametrize("username,password,verification_code", read_csv_data())
def test_login_from_csv(driver, username, password, verification_code):
    """
    从 CSV 文件读取并测试登录情况
    """
    logger.info(f"'----------' 开始执行测试 '----------'\n")
    perform_login(driver, username, password, verification_code)
    logger.info(f"'----------' 执行完成测试 '----------'\n")
    driver.implicitly_wait(10)

if __name__ == '__main__':
    pytest.main()
